/**********************************************************************
  ブレークポイントの設定
  $Id: bp_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"

int  bp_parse_opt(struct GETOPT_ARG *args) ;
void bp_print_help() ;

int bp_cmd( struct GETOPT_ARG *args , int mode )
{
  int i,j ;
  int id ;
  char *label ;
  int ret ;
  
  ret=bp_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  if( XGETOPT_REFARGP(args,1)==NULL ){
    for( i=0 ; i<MAX_PROCESS_ID ; i++ ){
      if( proc_list.proc[i].bp_num>0 ){
	SYSMSG("--- Break Point(s) for Process ID = %2d --\n",i) ;
	for( j=0 ; j<proc_list.proc[i].bp_num ; j++ ){
	  SYSMSG("No.%02d : 0x%08x %s\n",
		 proc_list.proc[i].bp[j].id,
		 proc_list.proc[i].bp[j].ad,
		 proc_list.proc[i].bp[j].label) ;
	}
      }
    }
    return(NORMAL_EXIT) ;
  }
  if( XGETOPT_REFARGP(args,2)==NULL ){
    id=proc_list.cur_id ;
  }else{
    if(       XGETOPT_REFARGP(args,2)[0]=='+' ){
      id=proc_list.cur_id+1 ;
    }else if( XGETOPT_REFARGP(args,2)[0]=='-' ){
      id=proc_list.cur_id-1 ;
    }else if( XGETOPT_REFARGP(args,2)[0]=='=' ){
      id=proc_list.cur_id ;
    }else{
      id=exatoi(XGETOPT_REFARGP(args,2)) ;
    }
    if( id>=MAX_PROCESS_ID || id<0 ){
      SYSMSG("Invalid Process ID (Min=0,Max=%d specify=%d).\n",
	     (MAX_PROCESS_ID)-1,id) ;
      return(NORMAL_EXIT) ;
    }
  }
  if( XGETOPT_REFARGP(args,3)!=NULL ){
    label=malloc( strlen(XGETOPT_REFARGP(args,3))+1 ) ;
    strcpy(label,XGETOPT_REFARGP(args,3)) ;
  }else{
    label=malloc( 1 ) ;
    label[0]='\0' ;
  }
  
  if( proc_list.proc[id].bp_num==MAX_BREAK_POINT_NUM ){
    SYSMSG("Break Point Buffer is full.\n") ;
    return(NORMAL_EXIT) ;
  }
  
  proc_list.proc[id].bp[proc_list.proc[id].bp_num].ad = exatoi(XGETOPT_REFARGP(args,1)) ;
  proc_list.proc[id].bp[proc_list.proc[id].bp_num].id = proc_list.proc[id].bp_id++ ;
  proc_list.proc[id].bp[proc_list.proc[id].bp_num].label=label ;
  SYSMSG("Setting bp=0x%08x , BPID=%d , label=%s , PID=%d\n",
	 proc_list.proc[id].bp[proc_list.proc[id].bp_num].ad,
	 proc_list.proc[id].bp[proc_list.proc[id].bp_num].id,
	 proc_list.proc[id].bp[proc_list.proc[id].bp_num].label,
	 id ) ;
  proc_list.proc[id].bp_num++ ;
  return(NORMAL_EXIT) ;

}


int bp_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      bp_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void bp_print_help()
{
  SYSERR(
	 "Set break point\n"
	 "Usage: bp [options] [adrs [[pid|+|-|=] label]]\n"
	 "  adrs  : break point adrs\n"
	 "  pid   : process ID\n"
	 "   +    : same current process ID+1\n"
	 "   -    : same current process ID-1\n"
	 "   =    : same current process ID\n"
	 "  label : break point label\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     ブレークポイントを設定します。設定できるブレークポイント数は1プロセスID\n"
	 "    につき %d 個まで指定できます。\n"
	 "     プロセスIDを省略した場合はカレントプロセスIDに対してブレークポイントを\n"
	 "    設定します。'+'を指定した場合はカレントプロセスID+1に、'-'を指定した場合\n"
	 "    はカレントプロセスID-1に、'='を指定した場合はカレントプロセスにブレーク\n"
	 "    ポイントを設定します。\n"
	 "     labelで文字列をブレークポイントのラベルとして指定できます。カレントプロ\n"
	 "    セスにlabelを指定する場合、'='を使用して明示的にカレントプロセスを指定す\n"
	 "    る必要があります。\n"
	 "\n"
	 "     ブレークアドレスの指定を省略した場合は、設定されている全ブレーク\n"
	 "    ポイントの一覧を表示します。\n"
	 , MAX_BREAK_POINT_NUM
	 ) ;
}
