/**********************************************************************
  コマンド解析/実行
  $Id: cmd_exec.c 79 2011-12-03 15:34:56Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "cmd_funcs.h"
#include "memory.h"
#include "sim_io.h"

#define	SCRIPT_MODE     0
#define	COMMAND_MODE    1
#define	STR_LINE_MAX    256

#ifdef HAVE_READLINE
#include <readline/readline.h>
#include <readline/history.h>
#endif

/**************************************
 内部コマンド名→関数呼び出しテーブル
**************************************/
static struct CMD_TBL_STRUCT cmd_tbl[] = {
#include	"cmd_tbl.h"
	{ NULL			, NULL , 0 }	/*** テーブル終端 ***/
} ;

int  help_parse_opt(struct GETOPT_ARG *args) ;
void help_print_help() ;

int	cmd_exec( char *file )
{
  struct TOKEN_STRUCT  token ;
  struct GETOPT_ARG    args ;
  FILE                *ifp ;
  static char          str_line[STR_LINE_MAX] ;
  int                  exec_mode ;
  static char          prompt[32] = "PPC> \0" ;
  int                  i,j ;
  int                  id ;
#ifdef HAVE_READLINE
  char                *rdline ;
#else
  int                  is_tty=0 ;
#endif
  
  ifp = stdin ;
  id=proc_list.cur_id ;
  if( *file=='-' ){
#ifdef HAVE_READLINE
    exec_mode = COMMAND_MODE ;
#else
    is_tty=isatty(fileno(ifp)) ;
    if( is_tty==1 ) exec_mode = COMMAND_MODE ;
    else            exec_mode = SCRIPT_MODE ;
#endif
  }else{
    if( (ifp = fopen( file , "rb" )) == NULL ){
      SYSMSG("Can not open file(%s).\n",file) ;
      return(FILE_OPEN_ERROR) ;
    }
    exec_mode = SCRIPT_MODE ;
  }
  if( exec_mode == SCRIPT_MODE ) SYSMSG("%s Exec Start \n",file) ;
  
  token.str=NULL ;
  
  while( -1 ){
    if( exec_mode == COMMAND_MODE ){
#ifdef HAVE_READLINE
      rdline = readline (prompt) ;
      if( rdline == NULL ){
	break ;
      }
      add_history(rdline) ;
      strncpy( str_line, rdline, STR_LINE_MAX ) ;
#else
      SYSMSG("%s",prompt ) ;
      if( fgets( str_line , STR_LINE_MAX , ifp )==NULL ) break ;
#endif
    }else{
      if( fgets( str_line , STR_LINE_MAX , ifp )==NULL ) break ;
      if( feof(ifp)!=0 ) break ;
      if( exec_mode == SCRIPT_MODE ) SYSMSG("%s",str_line ) ;
    }
    
    get_token( str_line , &token ) ;
    
    if( token.num<0 ){
      SYSMSG("PPCSIM internal error.\n") ;
      return( INTERNAL_ERROR ) ;
    }
    
    if( token.num==0 ) continue ;
    
    /*** exitコマンドの実行 ***/
    if( strcmp((char*)"exit", token.pt[0]) == 0 ){
      break ;
    }
    
    /*** コマンド実行 ***/
    XGETOPT_INIARG(args) ;
    for( i=0 ; i<token.num ; i++ ){
      XGETOPT_ADDARG(args,token.pt[i]) ;
    }
    for( i=0 ; ; i++ ){
      if( cmd_tbl[i].cmd_name == NULL ){ /*** 外部コマンドの実行 ***/
	if( XGETOPT_REFAGC(args) !=0 ){
	  for( j=0 ; ; j++ ){
	    if( str_line[j]!=' ' && str_line[j]!='\t' ) break ;
	  }
	  if( str_line[j]=='_' ){
	    if( str_line[j+1]=='\0' ||	/* ←これはあり得ない? */
		str_line[j+1]=='\r' || str_line[j+1]=='\n' ){
	      SYSMSG("command not found.\n") ;
	      break ;
	    }
	    j++ ;
	  }
	  system( &str_line[j] ) ;
	}
	break ;
      }
      if( strcmp( cmd_tbl[i].cmd_name, XGETOPT_REFARG(args,0)) == 0 ){ /*** 内部コマンドの実行 ***/
	if( proc_list.proc[id].mem == NULL ){/*** 実行メモリ割り当てチェック ***/
	  SYSMSG("Memory for simulate is not allocated .\n" ) ;
	  break ;
	}
	(*cmd_tbl[i].func)( &args , cmd_tbl[i].mode ) ;
	break ;
      }
    }
    XGETOPT_FREARG(args) ;
  }
  if( exec_mode == SCRIPT_MODE ) SYSMSG("done.\n") ;
  return( NORMAL_EXIT ) ;
}


int help_cmd( struct GETOPT_ARG *args , int mode )
{
  int i,j ;
  struct GETOPT_ARG args_tmp ;
  int ret ;
  
  ret=help_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;

  if( XGETOPT_REFAGCP(args)==1 ){
    SYSMSG("List of builtin commands\n") ;
    for( i=0 ; ; i++ ){
      if( cmd_tbl[i].cmd_name==NULL ) break ;
      SYSMSG("%-7s : %-25s",cmd_tbl[i].cmd_name,cmd_tbl[i].expo) ;
      if( i%2==0 ){
	SYSMSG(", ") ;
      }else{
	SYSMSG("\n") ;
      }
    }
    SYSMSG("\n") ;
  }else{
    for( i=1 ; ; i++ ){
      if( XGETOPT_REFARGP(args,i)==NULL ) break ;
      for( j=0 ; ; j++ ){
	if( cmd_tbl[j].cmd_name==NULL ){
	  SYSMSG("%s is no match builtin commands\n", XGETOPT_REFARGP(args,i)) ;
	  break ;
	}
	if( strcmp(XGETOPT_REFARGP(args,i),cmd_tbl[j].cmd_name)==0 ){
	  XGETOPT_INIARG(args_tmp) ;
	  XGETOPT_ADDARG(args_tmp,XGETOPT_REFARGP(args,i)) ;
	  XGETOPT_ADDARG(args_tmp,"--help") ;
	  (*cmd_tbl[j].func)( &args_tmp , cmd_tbl[j].mode ) ;
	  XGETOPT_FREARG(args_tmp) ;
	  break ;
	}
      }
    }
  }
  return(NORMAL_EXIT) ;
}


int help_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      help_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void help_print_help()
{
  SYSERR(
	 "Print help for builtin commands\n"
	 "Usage: help [Options] [command]\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     commandで指定したビルトインコマンドのヘルプを表示します。\n"
	 "    省略した場合はビルトインコマンドの一覧を表示します。\n"
	 ) ;
}
