/**********************************************************************
  sim DISK (Pseudo DISK device)
  $Id: dev_disk.h 77 2011-11-25 15:16:46Z tane $
**********************************************************************/

/*** DISK device ***/
#define DISK_CMD             0x8000ff10
#define DISK_ACK             0x8000ff14
#define DISK_DRIVE           0x8000ff18
#define DISK_TRACK           0x8000ff1c
#define DISK_DATA_TOP        0x8000ff20
#define DISK_DATA_LEN        0x8000ff24

#define DISK_INQ_LEN         256

#define DISK_NOP             0x00
#define DISK_INQ             0x01
#define DISK_READ            0x02
#define DISK_WRITE           0x03
#define DISK_WRITE_INQ       0x04

#define DISK_ACK_IDLE        0x00
#define DISK_ACK_BUSY        0x01
#define DISK_ACK_FAIL        0x81

#define SIM_IO_DISK_MOUNT_FAIL  0x00000001

#define VENDER_ID_LEN      32
#define DISK_INFO_PADING  208
#define VENDER_ID         "SIMLATION_DISK"

struct sim_disk_info_struct{
  unsigned char vender_id[VENDER_ID_LEN] ; /** ベンダーID                 32byte **/
  int  type     ;                          /** DISK TYPE                   4byte **/
  int  tnum     ;                          /** トラック数                  4byte **/
  int  tlen     ;                          /** 1トラックあたりのバイト数   4byte **/
  int  reserve1 ;                          /** パディング                  4byte **/
  unsigned char pading[DISK_INFO_PADING] ; /** 未使用領域                208byte **/
} ;

#define SIM_DISK_DRIVE_NUM   16

struct sim_disk_struct{
  int mount ;
  int type ;
  int tnum ;
  int tlen ;
  struct sim_disk_info_struct *info ;
  unsigned char *buf ;
} ;
struct sim_disk_struct simDisk[SIM_DISK_DRIVE_NUM] ;

#define DISK_TYPE_EN_READ     0x00000001
#define DISK_TYPE_EN_WRITE    0x00000002
#define DISK_TYPE_EN_EJECT    0x00000004

int dev_disk( unsigned int port , struct MACHINE_RESOURCE *mres ) ;
int dev_disk_init( struct MACHINE_RESOURCE *mres ) ;

/*********************************************************************
============================
  DISK
============================
  DISK_CMD_ADRS にコマンドを書きこむ事でディスクデバイスをアクセスする。

     DISK_READ
       DISK_DRIVEで指定されたドライブからDISK_TRACKで指定したトラック
       をDSIK_DATA_TOPで指定した物理メモリからDISK_DATA_LENバイト読み
       こみます。
  
     DISK_WRITE
       DISK_DRIVEで指定されたドライブからDISK_TRACKで指定したトラック
       をDSIK_DATA_TOPで指定した物理メモリにDISK_DATA_LENバイト書きこ
       みます。

     DISK_INQ
       ディスクの情報をDISK_DATA_TOPで指定した物理メモリにDISK_INQ_LEN
       バイト書きこみます。データ構造の実体はstruct sim_disk_info_struct
       そのもの。ただしエンディアンはビッグエンディアン。

   DISK_ACK
     アクセスの結果を返します。
     →DMAをシミュレートして割り込みをトリガに処理できるようにするべき。

 *********************************************************************/
