/**********************************************************************
  disassamble コマンド
  $Id: dis_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"

int  dis_parse_opt(struct GETOPT_ARG *args) ;
void dis_print_help() ;

int dis_cmd( struct GETOPT_ARG *args , int mode )
{
  int i ;
  static int start_adrs=0  ;
  static int leng      =20 ;
  struct MACHINE_RESOURCE tmp_proc ;
  int ret ;
  
  ret=dis_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  if( XGETOPT_REFARGP(args,1)!=NULL ){
    start_adrs=exatoi(XGETOPT_REFARGP(args,1)) ;
  }
  if( XGETOPT_REFARGP(args,2)!=NULL ){
    leng=exatoi(XGETOPT_REFARGP(args,2)) ;
  }
  tmp_proc=proc_list.proc[proc_list.cur_id] ;

  tmp_proc.pc=start_adrs ;
  for( i=0 ; i<leng ; i++ ){
    SYSMSG("%3d : %08x : ",proc_list.cur_id,tmp_proc.pc) ;
    
    op_exec(&tmp_proc,EXEC_DIS) ;
    tmp_proc.pc+=4 ;
  }
  start_adrs+=(leng*4) ;
 
  return(NORMAL_EXIT) ;
}


int dis_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      dis_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void dis_print_help()
{
  SYSERR(
	 "Disassemble command\n"
	 "Usage: dis [adrs [lines]]\n"
	 "  adrs  : Disassemble start address.\n"
	 "  lines : Number of listing lines.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     disassembleリストを表示します。開始アドレス及び表示行数を省略した場合、\n"
	 "    一つ前に実行したdisの 開始アドレス+表示行数*4byte からdisassembleリスト\n"
	 "    を表示します。一番最初に開始アドレスを省略した場合は0x00000000 、表示\n"
	 "    行数は20が省略値となります。\n"
	 ) ;
}
