/**********************************************************************
  DISKデバイスへのファイルロードコマンド
  $Id: dload_cmd.c 77 2011-11-25 15:16:46Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "sim_io.h"

int  dload_parse_opt(struct GETOPT_ARG *args) ;
void dload_print_help() ;

int dload_cmd( struct GETOPT_ARG *args , int mode )
{
  int    drv ;
  int    res ;
  FILE   *fp;
  int    ret ;

  ret=dload_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  res=NORMAL_EXIT ;

  if( XGETOPT_REFARGP(args,1)==NULL ){
    SYSMSG("Specified invalid file name.\n") ;
    res=SPECIFY_ERROR ;
  }else{
    drv=0 ;
    if( XGETOPT_REFARGP(args,2)!=NULL ){
      drv=exatoi(XGETOPT_REFARGP(args,2)) ;
      if( drv<0 || drv>=SIM_DISK_DRIVE_NUM ){
	SYSMSG("Specified invalid drive number.\n") ;
	res=SPECIFY_ERROR ;
      }else{
	if( simDisk[drv].mount!=VALID ){
	  SYSMSG("DISK %i is not mounted.\n",drv) ;
	  res=SPECIFY_ERROR ;
	}else{
	  fp=fopen( XGETOPT_REFARGP(args,1), "rb" ) ;
	  if( fp==NULL ){
	    SYSMSG("Can not open file %s.\n",XGETOPT_REFARGP(args,1)) ;
	  }else{
	    fread( simDisk[drv].buf, sizeof(unsigned char),
		   (simDisk[drv].tnum * simDisk[drv].tlen) , fp ) ;
	  }
	}
      }
    }
  }

  return(res) ;
}


int dload_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      dload_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void dload_print_help()
{
  SYSERR(
	 "Load DISK image to DISK device\n"
	 "Usage: dload file drv\n"
	 "  file  : file name\n"
	 "  drv   : Drive number(0 to %d)\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     mountコマンドでマウントしたディスクデバイスにディスクデバイスイメージ\n"
	 "    のファイルをロードします。\n"
	 "    ※ここでのファイルはppcsimが動作しているOS上のファイルシステムのことです\n"
	 , SIM_DISK_DRIVE_NUM-1
	 ) ;
}
