/**********************************************************************
  各種値のダンプコマンド
  $Id: dump_cmd.c 79 2011-12-03 15:34:56Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"

int  dump_parse_opt(struct GETOPT_ARG *args) ;
void dump_print_help() ;

int dump_cmd( struct GETOPT_ARG *args , int mode )
{
  struct	TARGET_TBL_STRUCT{
    char    *name ;
    int     flag ;
  } ;
  static struct TARGET_TBL_STRUCT trg_tbl[] = {
    { "reg"		, (PRINT_CRS|PRINT_GPRS) },
    { "r"		, (PRINT_CRS|PRINT_GPRS) },
    { "all"		, (PRINT_CRS|PRINT_GPRS|PRINT_FPRS) },
    { "a"		, (PRINT_CRS|PRINT_GPRS|PRINT_FPRS) },
    { "spr"		, (PRINT_SPRS)           },
    { "s"		, (PRINT_SPRS)           },
    { "tr"		, (PRINT_CRS|PRINT_TRS)  },
    { "t"		, (PRINT_CRS|PRINT_TRS)  },
    { "mem"		, (PRINT_MEMORYDUMP)     },
    { "m"		, (PRINT_MEMORYDUMP)     },
    { NULL		, 0 }		/*** テーブル終端 ***/
  } ;
  static int    start_adrs=0 ;
  static int    leng=(8*16) ;
  int    i ;
  int    flag ;
  int    id ;
  int    ret ;
  
  ret=dump_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  id=proc_list.cur_id ;
  if( XGETOPT_REFARGP(args,1)==NULL ){
    SYSMSG("Too few argument(s)\n") ;
    return(SPECIFY_ERROR) ;
  }
  
  for( i=0 ; ; i++ ){
    if( trg_tbl[i].name==NULL ){
      SYSMSG("Specified invalid argument(%s)\n",XGETOPT_REFARGP(args,1)) ;
      return(SPECIFY_ERROR) ;
    }
    if( strcmp(XGETOPT_REFARGP(args,1),trg_tbl[i].name)==0 ){
      flag=trg_tbl[i].flag ;
      break ;
    }
  }
  
  if( (flag & PRINT_MEMORYDUMP)==0 ){
    return( print_regs( &proc_list.proc[id], flag ) ) ;
  }else{
    if( XGETOPT_REFARGP(args,2)!=NULL ){
      start_adrs=exatoi(XGETOPT_REFARGP(args,2)) ;
    }
    if( XGETOPT_REFARGP(args,3)!=NULL ){
      i=exatoi(XGETOPT_REFARGP(args,3)) ;
      if( i<0 ) ;
      else leng=((i/16)+((i%16)!=0))*16 ;
    }
    {
      int ret ;
      ret=print_mems( &proc_list.proc[id], start_adrs, leng ) ;
      if( ret==NORMAL_EXIT ) start_adrs += leng ;
      return( ret ) ;
    }
  }
  
  return(NORMAL_EXIT) ;
}


int dump_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      dump_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void dump_print_help()
{
  SYSERR(
	 "Dump registers and memory\n"
	 "Usage: dump [options] type [adrs]\n"
	 " type\n"
	 "  reg or r : dump GPR and other registers.\n"
	 "  all or a : dump GPR and FPR and other registers.\n"
	 "  tr  or t : dump GPR and TR and other registers.\n"
	 "  spr or s : dump all SPRs.\n"
	 "  mem or m : dump memory.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     regを指定した場合、GPR その他 レジスタの一覧を表示します。\n"
	 "     allを指定した場合、GPRに加え FPRの 一覧を表示します。\n"
	 "     tr を指定した場合、アドレス変換レジスタ群の一覧を表示します。\n"
	 "     sprを指定した場合、全SPR 一覧を表示します。\n"
	 "     memを指定した場合シミュレータメモリの内容を表示します。表示開始アドレス\n"
	 "    を省略した場合、一つ前に実行したdumpの 表示開始アドレス+0x80 から0x80バ\n"
	 "    イトを表示します。一番最初に表示開始アドレスを省略した場合は、0x00000000\n"
	 "    が表示開始アドレスになります。\n"

	 ) ;
}
