/**********************************************************************
  バイナリファイル実行コマンド
  $Id: exec_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "funcs.h"
#include "processor.h"
#include "memory.h"
#include "op_exec.h"
#include "sim_io.h"

int  exception( struct MACHINE_RESOURCE *mres , int code ) ;
int  exec_parse_opt(struct GETOPT_ARG *args) ;
void exec_print_help() ;
int  continue_parse_opt(struct GETOPT_ARG *args) ;
void continue_print_help() ;

int break_on_start ;

int exec_cmd( struct GETOPT_ARG *args , int mode )
{
  int excp ;
  int id ;
  int res ;
  int break_flag ;
  int exec_mode ; 
  int ret ;

  break_on_start=NO ;
  if( strcmp("c",XGETOPT_REFARGP(args,0))==0 ){
    ret=continue_parse_opt(args) ;
  }else{
    ret=exec_parse_opt(args) ;
  }
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  break_flag=ON ;
  id=proc_list.cur_id ;
  
  if( (mode & GO_EXEC_MODE)!=0 ){ //新規プロセスをforkして実行する 
    res=create_proc( &proc_list, default_textsize ) ;
    if( res != NORMAL_EXIT ){
      SYSMSG("Failed to create process.\n") ;
      return( res ) ;
    }
    id=proc_list.cur_id ;
    res=elf_load(  &proc_list.proc[id], XGETOPT_REFARGP(args,1) ) ;
    if( res != NORMAL_EXIT ){
      remove_proc( &proc_list ) ;
      return( res ) ;
    }
    
    res=sim_setarg( &proc_list.proc[id], args, 1 ) ;
    if( res != NORMAL_EXIT ){
      if( res==SETARG_TOO_MANY_ARGS ){
	SYSMSG("too many arguments.\n") ;
      }
      remove_proc( &proc_list ) ;
      return( res ) ;
    }
    do{
      int i ;
      int leng ;
      for( i=0 ; i<(PNAME_LEN+1) ; i++ ) proc_list.proc[id].pname[i]='\0' ;
      leng=strlen(XGETOPT_REFARGP(args,1))+1 ;
      if( leng>=PNAME_LEN ) leng=PNAME_LEN ;
      strncpy( proc_list.proc[id].pname, XGETOPT_REFARGP(args,1), leng ) ;
    } while(0) ;
    
    iexe_count  = 0 ;
    break_flag=OFF ;

    if( break_on_start==YES ){
      return(NORMAL_EXIT) ;
    }
  }
  
  set_sighandl_ppcsim() ;
  while ( -1 ) {
    exec_mode=EXEC_INST ;
    
    if( sig_int != 0 ) break ;
    
    if( trace_stat.msg_type!=TRACE_OFF ){
      exec_mode |= EXEC_DIS ;
      print_regs( &proc_list.proc[id], trace_stat.print_type ) ;
    }
    
    if( proc_list.proc[id].bp_num > 0 ){
      int i ;
      for( i=0 ; i<proc_list.proc[id].bp_num ; i++ ){
	if( proc_list.proc[id].pc==proc_list.proc[id].bp[i].ad ){
	  if( break_flag==ON ){ //breakポイントで止まり再開した時にまた引っかからない様にする
	    break ;
	  }else{
	    SYSMSG("Exec stop by break point.\n") ;
	    SYSMSG("%3d : %08x : ",id,proc_list.proc[id].pc) ;
	    op_exec(&proc_list.proc[id],EXEC_DIS) ;
	    return(NORMAL_EXIT) ;     
	  }
	}
      }
      break_flag=OFF ;
    }
    
    if( proc_list.proc[id].pp_num > 0 ){
      int i ;
      for( i=0 ; i<proc_list.proc[id].pp_num ; i++ ){
	if( proc_list.proc[id].pc==proc_list.proc[id].pp[i].ad ){
	  SYSMSG("Passed count= ") ;
	  SYSMSGLL(iexe_count,0) ;
	  SYSMSG("\n") ;
	  switch( ppmsg_stat.msg_type ){
	  case PP_DUMP_PC :
	    exec_mode |= EXEC_DIS ;
	    break ;
	  default :
	    print_regs( &proc_list.proc[id], ppmsg_stat.print_type ) ;
	    break ;
	  }
	  break ;
	}
      }
    }

    if( (exec_mode & EXEC_DIS)!=0 ){
      SYSMSG("%3d : %08x : ",id,proc_list.proc[id].pc) ;
    }
    excp = op_exec( &proc_list.proc[id], exec_mode ) ;

    trace.buf[trace.ptr].pid  = id ;
    trace.buf[trace.ptr].pc   = proc_list.proc[id].pc ;
    trace.buf[trace.ptr].inst = proc_list.proc[id].inst ;
    trace.ptr=(trace.ptr+1)%trace.size ;
    
    iexe_count++ ;
    
    sim_io( &proc_list.proc[id] ) ;
    
    if( excp !=0 ){
      switch( excp ){
      case SYSCALLED_EXIT :
	remove_proc( &proc_list ) ;
	id=proc_list.cur_id ;
	if( id==0 ){
	  return(proc_list.proc[id].GPR[3].rl) ;
	}
	proc_list.proc[id].GPR[3].rl= 1 ;
	break ;
      case SYSCALLED_FORK :
	proc_list.proc[id].GPR[3].rl= 1 ; /** return value of parent process */
	res=sim_fork( &proc_list ) ;
	if( res!=NORMAL_EXIT ){
	  SYSMSG("fork() faild.\n") ;
	  print_regs( &proc_list.proc[id], PRINT_CRS|PRINT_GPRS ) ;
	  return(res) ;
	}
	id=proc_list.cur_id ;
	proc_list.proc[id].GPR[3].rl= 0 ; /** return value of child process */
	proc_list.proc[id].id       = id ;
	break ;
      case SYSCALLED_EXECVE :
	res=sim_execve( &proc_list.proc[id] ) ;
	if( res!=NORMAL_EXIT ){
	  SYSMSG("execve() faild.") ;
	  if( res==SETARG_TOO_MANY_ARGS ){
	    SYSMSG("too many arguments.") ;
	  }
	  SYSMSG("\n") ;
	  print_regs( &proc_list.proc[id], PRINT_CRS|PRINT_GPRS ) ;
	  return(res) ;
	}
	break ;
      case MEM_ACCESS_FAULT :
	SYSMSG("Memory access fault in syscall emulation.\n") ;
	print_regs( &proc_list.proc[id], PRINT_CRS|PRINT_GPRS ) ;
	return(NORMAL_EXIT) ;
	break ;
      default :
	excp=exception( &proc_list.proc[id], excp ) ;
	if( excp!=0 ) return(NORMAL_EXIT) ;
      }
    }
  }
  return(NORMAL_EXIT) ;
}


int exec_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"break",              'b',  NONARG, 'b'},
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  int arg0 ;

  arg0=0 ;
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'b' :
      break_on_start=YES ;
      break ;
    case 'h' :
      exec_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      if( arg0==0 ){
	arg0++ ;
      }else{
	return(0) ;
      }
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


int continue_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      continue_print_help() ;
      return(-1) ;
      break ;
    case 'b' :
      continue_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void exec_print_help()
{
  SYSERR(
	 "Loading ELF file and execute\n"
	 "Usage: exec [Options] file [arg1 arg2 ...]\n"
	 " Options\n"
	 "  -b or --break : 実行ファイルロード後、プロセスを停止します\n"
	 "  -h or --help  : Print this help.\n"
	 "\n"
	 "     ELF形式の実行ファイルをロードし実行します。\n"
	 ) ;
}


void continue_print_help()
{
  SYSERR(
	 "Continue execution\n"
	 "Usage: c [Options]\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     ブレークポイントで設定されたアドレスの命令を実行して停止したときの実行\n"
	 "    再開を行います。goコマンドのアドレス省略による再開と異なる点は、cコマンド\n"
	 "    では、exit() システムコールを実行するとプロセスも消滅します。\n"
	 "\n"
	 "     実行時のレジスタ表示などはtraceコマンドで設定した条件に従います。\n"
	 ) ;
}
