/**********************************************************************
  実行コマンド
  $Id: go_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "funcs.h"
#include "processor.h"
#include "memory.h"
#include "op_exec.h"
#include "sim_io.h"

int	op_external_interrupt( struct MACHINE_RESOURCE *mres ) ;
int	op_decrement( struct MACHINE_RESOURCE *mres ) ;
int	op_single_step( struct MACHINE_RESOURCE *mres ) ;
int	chk_single_step( struct MACHINE_RESOURCE *mres ) ;
int	exception( struct MACHINE_RESOURCE *mres , int code ) ;
int     go_parse_opt(struct GETOPT_ARG *args) ;
void    go_print_help() ;
int     step_parse_opt(struct GETOPT_ARG *args) ;
void    step_print_help() ;

int	go_cmd( struct GETOPT_ARG *args , int mode )
{
  int	excp ;
  int	i,exec_num ;
  int	break_flag ;
  int     id ;
  int     do_single_step ;
  int     exec_mode ; 
  static	int step_num=1 ;
  int ret ;
  
  if( strcmp("g",XGETOPT_REFARGP(args,0))==0 ||
      strcmp("go",XGETOPT_REFARGP(args,0))==0 ){
    ret=go_parse_opt(args) ;
  }else{
    ret=step_parse_opt(args) ;
  }
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  id=proc_list.cur_id ;
  break_flag=ON ;
  //現在のpcがbreak_ptと同じ場合は疑似的にブレークポイントを無効にする
  if( proc_list.proc[id].bp_num > 0 ){
    for( i=0 ; i<proc_list.proc[id].bp_num ; i++ ){
      if( proc_list.proc[id].pc==proc_list.proc[id].bp[i].ad ){
	break_flag=OFF ;
	break ;
      }
    }
  }
  
  if( (mode & GO_STEP_MODE)==0 ){
    exec_num=-1 ;
  }else{
    if( XGETOPT_REFARGP(args,1)!=NULL ){
      exec_num=exatoi( XGETOPT_REFARGP(args,1) ) ;
      if( exec_num>0 ) step_num=exec_num ;
      else{
	SYSMSG("Invalid step number. range=1-%d\n",0x7fffffff) ;
	return(NORMAL_EXIT) ;
      }
    }
    exec_num=step_num ;
  }
  
  set_sighandl_ppcsim() ;
  while ( -1 ) {
    RST_MEM_PROBE() ;
    exec_mode=EXEC_INST ;
    
    if( sig_int != 0 ) break ;
    
    excp = op_decrement( &proc_list.proc[id] ) ; // occurd declement exception
    if( excp !=0 ){
      excp=exception( &proc_list.proc[id], excp) ;
      if( excp!=0 ) break ;
    }
    
    excp = op_external_interrupt( &proc_list.proc[id] ) ; // occurd declement exception
    if( excp !=0 ){
      excp=exception( &proc_list.proc[id], excp) ;
      if( excp!=0 ) break ;
    }
    
    if( break_flag==ON && proc_list.proc[id].bp_num > 0 ){
      for( i=0 ; i<proc_list.proc[id].bp_num ; i++ ){
	if( proc_list.proc[id].pc==proc_list.proc[id].bp[i].ad ){
	  break_flag=OFF ;
	  SYSMSG("Exec stop by break point.  : %s\n",
		 proc_list.proc[id].bp[i].label ) ;
	  op_exec(&proc_list.proc[id],EXEC_DIS) ;
	  return(NORMAL_EXIT) ;
	}
      }
    }
    
    if( proc_list.proc[id].pp_num > 0 ){
      for( i=0 ; i<proc_list.proc[id].pp_num ; i++ ){
	if( proc_list.proc[id].pc==proc_list.proc[id].pp[i].ad ){
	  SYSMSG("Passed count= ") ;
	  SYSMSGLL(iexe_count,0) ;
	  SYSMSG(" : %s",proc_list.proc[id].pp[i].label) ;
	  SYSMSG("\n") ;
	  switch( ppmsg_stat.msg_type ){
	  case PP_DUMP_PC :
	    exec_mode |= EXEC_DIS ;
	    break ;
	  default :
	    print_regs( &proc_list.proc[id], ppmsg_stat.print_type ) ;
	    break ;
	  }
	  break ;
	}
      }
    }
    
    break_flag=ON ;
    
    if( trace_stat.msg_type!=TRACE_OFF ){
      exec_mode |= EXEC_DIS ;
      print_regs( &proc_list.proc[id], trace_stat.print_type ) ;
    }
    
    do_single_step=chk_single_step( &proc_list.proc[id] ) ;
    
    if( (exec_mode & EXEC_DIS)!=0 ){
      SYSMSG("%3d : %08x : ",id,proc_list.proc[id].pc) ;
    }
    excp = op_exec( &proc_list.proc[id], exec_mode ) ;
    
    trace.buf[trace.ptr].pid  = id ;
    trace.buf[trace.ptr].pc   = proc_list.proc[id].pc ;
    trace.buf[trace.ptr].inst = proc_list.proc[id].inst ;
    trace.ptr=(trace.ptr+1)%trace.size ;
    
    iexe_count++ ;
    
    if( CHK_MEM_PROBE() ){
      for( i=0 ; i<MemprobeBuf.num ; i++ ){
	if( (MemprobeBuf.adrs[i] <= __ea_for_exception) &&
	    (MemprobeBuf.adrs[i]+MemprobeBuf.size[i] > __ea_for_exception) ){
	  SYSMSG("touch memory ea:0x%08x pa:0x%08x (prob=0x%08x) at pc=0x%08x\n",
		 __ea_for_exception,__pa_for_exception,
		 MemprobeBuf.adrs[i], __mem_probe_pc ) ;
	  break ;
	}
      }
    }
    
    sim_io( &proc_list.proc[id] ) ;
    
    if( excp !=0 ){
      excp=exception( &proc_list.proc[id], excp ) ;
      if( excp!=0 ) break ;
    }
    
    if( do_single_step==YES ){
      excp = op_single_step( &proc_list.proc[id] ) ; // occurd single step exception
      if( excp !=0 ){
	excp=exception( &proc_list.proc[id], excp ) ;
	if( excp!=0 ) break ;
      }
    }
    
    exec_num-- ;
    if( exec_num==0 ) break ;
    else if( exec_num<0 ) exec_num=-1 ;
  }
  
#ifdef SUPPORT_GRAPHIC_DEVICE
  SyncGdevDraw(&proc_list.proc[id]) ;
#endif
  
  return(NORMAL_EXIT) ;
}


int go_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      go_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


int step_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      step_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void go_print_help()
{
  SYSERR(
	 "exec start command\n"
	 "Usage: go/g [Options]\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     ブレークポイントで設定されたアドレスの命令を実行して停止したときの実行\n"
	 "    再開を行います。goコマンドのアドレス省略による再開と異なる点は、cコマンド\n"
	 "    では、exit() システムコールを実行するとプロセスも消滅します。\n"
	 "\n"
	 "     実行時のレジスタ表示などはtraceコマンドで設定した条件に従います。\n"
	 ) ;
}


void step_print_help()
{
  SYSERR(
	 "step execution\n"
	 "Usage: step/s [Options] [num]\n"
	 "  num : execution instruction count\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     ステップ実行を行います。実行する命令数を省略した場合は、一つ前に実行し\n"
	 "    たstepコマンドの命令数と同じ命令数だけ実行します。一番最初に実行する時\n"
	 "    省略した場合は 1が指定されたのと等価になります。\n"
	 "\n"
	 "     実行時のレジスタ表示などはtraceコマンドで設定した条件に従います。\n"
	 ) ;
}


