/**********************************************************************
  マシンメモリイメージのファイルロードコマンド
  $Id: load_cmd.c 77 2011-11-25 15:16:46Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"

int  load_parse_opt(struct GETOPT_ARG *args) ;
void load_print_help() ;

int load_cmd( struct GETOPT_ARG *args , int mode )
{
  int    begin,size ;
  int    id ; 
  FILE   *fp;
  int    ret ;
  
  ret=load_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  id=proc_list.cur_id ;
  
  if( XGETOPT_REFAGCP(args)<2 ){
    SYSMSG("Too few argument(s)\n") ;
    return(SPECIFY_ERROR) ;
  }
  
  switch( (XGETOPT_REFAGCP(args)-1) ){
  case 2: /** file begin **/
    begin=exatoi(XGETOPT_REFARGP(args,2)) ;
    size =proc_list.proc[id].mem_size[seg(begin)]-lad(begin) ;
    break ;
  case 3: /** file begin size **/
    begin=exatoi(XGETOPT_REFARGP(args,2)) ;
    size =exatoi(XGETOPT_REFARGP(args,3)) ;
    break ;
  default :
    begin=0 ;
    size =proc_list.proc[id].mem_size[seg(begin)] ;
  }
  if( lad(begin)>=proc_list.proc[id].mem_size[seg(begin)] ){
    SYSMSG("Start address is invalid(addr=%d).\n", begin ) ;
    return(SPECIFY_ERROR) ;
  }
  if( lad(begin)+size > proc_list.proc[id].mem_size[seg(begin)] ){
    size = size - (begin+size-proc_list.proc[id].mem_size[seg(begin)]) ;
  }
  if( verbose==ON ) SYSMSG("begin=%d size=%d\n",begin,size) ;
  
  if( ( fp = fopen( XGETOPT_REFARGP(args,1) , "rb" )) == NULL ){
    SYSMSG("Can not open file(%s)\n",XGETOPT_REFARGP(args,1)) ;
    return(FILE_OPEN_ERROR) ;
  }
  fread( &proc_list.proc[id].mem[seg(begin)][lad(begin)] , size , 1, fp ) ;
  
  fclose( fp ) ;
  return(NORMAL_EXIT) ;
 
}


int load_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      load_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void load_print_help()
{
  SYSERR(
	 "Load file to memory\n"
	 "Usage: load [Options] file [adrs [size]]\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     シミュレータメモリ上にファイルをロードします。ロード開始番地を省略した\n"
	 "    場合は 0x00000000番地が開始番地になります。ロードサイズを省略した場合は\n"
	 "    ファイルサイズ分読み込みますが、メモリサイズ-先頭アドレスがファイルサイ\n"
	 "    ズより小さい場合はファイルの先頭からメモリサイズ分が読み込まれます。\n"
	 ) ;
}
