/**********************************************************************
  PPCネイティブ命令セット シミュレータ
  $Id: main.c 77 2011-11-25 15:16:46Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"
#include "op_scemu.h"
#include "cmd_funcs.h"
#include "version.h"

#ifdef __human__
#define	PLATFORM	"X68k"
#else
#ifdef __CYGWIN__
#define	PLATFORM	"Cygwin"
#else
#define	PLATFORM	"AnyHost"
#endif
#endif

#define RDFILE 1
#define RDARG  2

int parse_opt(struct GETOPT_ARG *args) ;
void manprint() ;

int main(int argc,char *argv[])
{
  struct GETOPT_ARG args ;
  int               ret ;
  int               i ;
  
  set_initparams() ;
  x_path = get_xpath( argv[0] ) ;

  XGETOPT_INIARG(args) ;
  for( i=1 ; i<argc ; i++ ){
    XGETOPT_ADDARG(args,argv[i]) ;
  }
  ret=parse_opt(&args) ;
  if( ret<0 ) goto err_exit ;

  //setmode( fileno(stdout),O_BINARY ) ;

  if( init_system()!=NORMAL_EXIT ) exit(0) ;

  if( XGETOPT_REFARG(args,0)==NULL ){ /*** interauctive mode ***/
    cmd_exec( "-\0" ) ;	
  }else if( ret==RDARG ){             /*** direct exec mode ***/
    XGETOPT_ADDTOPARG(args,"exec") ;
    XGETOPT_RESETFLAG(args) ;
    exit( exec_cmd( &args, GO_EXEC_MODE ) ) ;
    
  }else{                              /*** script-file exec mode ***/
    for( i=0 ; ; i++ ){
      char *fname ;
      fname=XGETOPT_REFARG(args,i) ;
      if( fname==NULL ) break ;
      cmd_exec( fname ) ;
    }
  }

  XGETOPT_FREARG(args) ;
  
  exit(0) ;

err_exit :
  exit(-1) ;
}

int parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    {"file",               'f',  NONARG, 'f'},
    {"outlog",             'o', NEEDARG, 'o'},
    {"text-memsize",       'M', NEEDARG, 'M'},
    {"heaptop-adrs",       'H', NEEDARG, 'H'},
    {"heap-memsize",       'P', NEEDARG, 'P'},
    {"exec-start-adrs",    'E', NEEDARG, 'E'},
    {"stack-top-offset",   'S', NEEDARG, 'S'},
    {"linux-sc-emu",       'L',  NONARG, 'L'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  
  int argst ;
  int ret ;
  
  simlog=NULL ;
  argst=RDARG ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      manprint() ;
      exit(0) ;
      break ;
    case 'f' :
      argst=RDFILE ;
      break ;
    case 'o' :
      simlog=malloc(strlen(xgetopt_val)+1) ;
      strcpy(simlog,xgetopt_val) ;
      break ;
    case 'M' :
      default_textsize=exatoi(xgetopt_val) ;
      break ;
    case 'H' :
      default_heaptop=exatoi(xgetopt_val) ;
      break ;
    case 'P' :
      default_heapsize=exatoi(xgetopt_val) ;
      break ;
    case 'E' :
      override_exec_start=YES ;
      default_exec_start=exatoi(xgetopt_val) ;
      break ;
    case 'S' :
      default_stack_top_offset=exatoi(xgetopt_val) ;
      break ;
    case 'L' :
      sc_emu_mode        = LINUXPPC ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      exit(-1) ;
    }
    if( ret==XGETOPT_NOT_OPTION ){
      if( argst==RDARG ) break ;
    }
  }
  return(argst) ;
}


void manprint()
{
  SYSERR("PPC Simulator for " PLATFORM " " VERSION " TANE\n"
	 "Usage: ppcsim [-Options] [cmd arg1 arg2 ....]\n"
	 " Options\n"
	 "  -M num or --text-memsize=num :\n"
	 "       Change default memory size for exec command & fork emulate\n"
	 "  -H num or --heaptop-adrs=num :\n"
	 "       Change default heap top for sbrk emulate\n"
	 "  -P num or --heap-memsize=num :\n"
	 "       Change default heap size for sbrk emulate\n"
	 "  -E num or --exec-start-adrs=num :\n"
	 "       Change default execute start address for exec command & fork emulate\n"
	 "  -S num or --stack-top-offset=num :\n"
	 "       Change default stack offset\n"
	 "  -L     or --linux-sc-emu :\n"
	 "       Change to Linux Syscall Emulation mode\n"
	 "  -o log or --outlog=log :\n"
	 "       Simulation log message output to file(default=stderr)\n"
	 "  -f ... or --file :\n"
	 "       Exec script file mode\n"
	 "  -h     or --help :\n"
	 "       Print this help\n"
	 "\n"
	 " if cmd or -f option is not specified, it start with interactive shell mode.\n") ;
}
