/**********************************************************************
  マシンメモリの割り当てコマンド
  $Id: memal_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"

int  memal_parse_opt(struct GETOPT_ARG *args) ;
void memal_print_help() ;

int memal_cmd( struct GETOPT_ARG *args, int mode )
{
  int   i ;
  int   seg ; 
  int   id ;
  int   res ;
  char *atrname[]={ "normal", "shared" } ;
  int   ret ;
  
  ret=memal_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  id=proc_list.cur_id ;
  
  if( XGETOPT_REFARGP(args,1)==NULL ){
    SYSMSG("---- Memory map of PID %d ---\n",id) ;
    for( i=0 ; i<MEM_SEGMENT_NUM ; i++ ){
      if( proc_list.proc[id].mem[i]!=NULL ){
	SYSMSG("%02x000000 : size=%08x , %s\n",
	       i,proc_list.proc[id].mem_size[i],
	       atrname[proc_list.proc[id].mem_attr[i]]) ;
      }else{
	//SYSMSG("%02x000000 : no allocate\n",i) ;
      }
    }
    return(NORMAL_EXIT) ;
  }
  
  if( XGETOPT_REFARGP(args,2)==NULL ) seg=0 ;
  else                                seg=exatoi(XGETOPT_REFARGP(args,2)) ;
  if( seg<0 || seg>=MEM_SEGMENT_NUM ) {
    SYSMSG("Specified invalid segment(range:0x00 to 0xff)\n") ;
    return(SPECIFY_ERROR) ;
  }
  
  i=exatoi(XGETOPT_REFARGP(args,1)) ;
  if( i>0 ){
    res=memal( &proc_list.proc[id], seg, i, MEM_ATTR_NORMAL ) ;
    if(       res==MEM_ALLOC_ERROR ){
      SYSMSG("Can not allocate memory(size=%d)\n", i ) ;
    }
    return( res ) ;
  }else{
    SYSMSG("Specified invalid memory size(range:0x00000001 to 0x01000000)\n") ;
    return(SPECIFY_ERROR) ;
  }
  
  return(NORMAL_EXIT) ;

}



int memal_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      memal_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void memal_print_help()
{
  SYSERR(
	 "Allocate simulation memory\n"
	 "Usage: memal [options] [size [seg]]\n"
	 "  size : 0x00000001 to 0x01000000\n"
	 "  seg  : 0x00 to 0xff\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     カレントプロセスの指定セグメントメモリサイズを変更します。変更前のメモリ\n"
	 "    の内容は引き継がれません。\n"
	 "     セグメントは0x00〜0xFFまでの範囲で指定します。一つのセグメントは最大16MB\n"
	 "    まで割り当てることができます。省略時は0x00が指定されたものとします。\n"
	 "    セグメントサイズより大きなメモリサイズを指定した時は連続したセグメント\n"
	 "    を順に割り当てながら指定したメモリサイズの割り当てを行います。\n"
	 "     引き数を省略すると、現在のメモリ割り当て状況が表示されます。\n"
	 ) ;
}

/*********************************************************************
  メモ
  020622
  セグメントサイズが256MBではサイズが大き過ぎてMPC106のメモリマッピング
  0xff______から割当たるROM空間の割り当てができなかったため、セグメント
  サイズを16MBまで縮小(セグメント数=256)する。
  また、メモリ割り当てを行う専用関数memal():memal.cを用意し、それを
  使用してメモリ割り当てを行う様に仕様変更する。
*********************************************************************/
