/**********************************************************************
  DISKデバイスマウント
  $Id: mount_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "funcs.h"
#include "processor.h"
#include "memory.h"
#include "op_exec.h"
#include "sim_io.h"

#define reverse_int(a)   (((a<<24)&0xff000000)|((a<<8)&0x00ff0000)|((a>>8)&0x0000ff00)|((a>>24)&0x000000ff))

#define MOUNT_MODE    0
#define UMOUNT_MODE   1

int  mount_parse_opt(struct GETOPT_ARG *args) ;
void mount_print_help() ;
int  umount_parse_opt(struct GETOPT_ARG *args) ;
void umount_print_help() ;

int mount_cmd( struct GETOPT_ARG *args , int mode )
{
  int i ;
  int drv, tnum, tlen ;
  int res ;
  int ret ;
  
  res=NORMAL_EXIT ;
  
  if( mode==MOUNT_MODE ){
    ret=mount_parse_opt(args) ;
    if( ret<0 ) return(NORMAL_EXIT) ;

    if( XGETOPT_REFARGP(args,1)==NULL ){
      for( i=0 ; i<SIM_DISK_DRIVE_NUM ; i++ ){
	if( simDisk[i].mount==VALID ){
	  SYSMSG("%3d  type=0x%08x track_len=0x%08x track_num=0x%08x\n",
		 i,
		 simDisk[i].type,
		 simDisk[i].tlen,
		 simDisk[i].tnum ) ;
	}
      }
    }else{
      drv=exatoi(XGETOPT_REFARGP(args,1)) ;
      if( drv<0 || drv>=SIM_DISK_DRIVE_NUM ){
	SYSMSG("Specified invalid drive number.\n") ;
	res=SPECIFY_ERROR ;
      }else{
	if( simDisk[drv].mount==VALID ){
	  SYSMSG("DISK %i is already mounted.\n",drv) ;
	  res=SPECIFY_ERROR ;
	}else{
	  if( XGETOPT_REFARGP(args,2)==NULL ){
	    SYSMSG("Specified invalid track length.\n") ;
	    res=SPECIFY_ERROR ;
	  }else{
	    tlen=exatoi(XGETOPT_REFARGP(args,2)) ;
	    if( tlen<=0 ){
	      SYSMSG("Specified invalid track length(%d).\n",tlen) ;
	      res=SPECIFY_ERROR ;
	    }else{
	      if( XGETOPT_REFARGP(args,3)==NULL ){
		SYSMSG("Specified invalid track number.\n") ;
		res=SPECIFY_ERROR ;
	      }else{
		tnum=exatoi(XGETOPT_REFARGP(args,3)) ;
		if( tnum<=0 ){
		  SYSMSG("Specified invalid track number(%d).\n",tnum) ;
		  res=SPECIFY_ERROR ;
		}else{
		  simDisk[drv].info=malloc( sizeof(struct sim_disk_info_struct) ) ;
		  if( simDisk[drv].info==NULL ){
		    SYSMSG("mount failed drive %d\n",drv) ;
		  }
		  simDisk[drv].buf =malloc( tnum * tlen ) ;
		  if( simDisk[drv].buf ==NULL ){
		    free( simDisk[drv].info ) ;
		    SYSMSG("mount failed drive %d\n",drv) ;
		  }else{
		    simDisk[drv].mount= VALID ;
		    simDisk[drv].type = ( DISK_TYPE_EN_READ | DISK_TYPE_EN_WRITE ) ;
		    simDisk[drv].tlen = tlen ;
		    simDisk[drv].tnum = tnum ;
		    memset( simDisk[drv].info->vender_id, 0, VENDER_ID_LEN ) ;
		    strcpy( (char*)simDisk[drv].info->vender_id , VENDER_ID ) ;
		    simDisk[drv].info->type = reverse_int( simDisk[drv].type ) ;
		    simDisk[drv].info->tnum = reverse_int( simDisk[drv].tnum ) ;
		    simDisk[drv].info->tlen = reverse_int( simDisk[drv].tlen ) ;
		    memset( simDisk[drv].info->pading, 0, DISK_INFO_PADING ) ;
		    memset( simDisk[drv].buf, 0, tlen * tnum ) ;
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }

  if( mode==UMOUNT_MODE ){
    ret=umount_parse_opt(args) ;
    if( ret<0 ) return(NORMAL_EXIT) ;

    if( XGETOPT_REFARGP(args,1)==NULL ){
      SYSMSG("Specified drive is not mounted.\n" ) ;
    }else{
      drv=exatoi(XGETOPT_REFARGP(args,1)) ;
      if( drv<0 || drv>=SIM_DISK_DRIVE_NUM ){
	SYSMSG("Specified invalid drive number.\n") ;
	res=SPECIFY_ERROR ;
      }else{
	if( simDisk[drv].mount != VALID ){
	  SYSMSG("Specific drive(%d) is not mounted .\n", drv ) ;
	  res=SPECIFY_ERROR ;
	}else{
	  simDisk[drv].mount= INVALID ;
	  simDisk[drv].type = 0 ;
	  simDisk[drv].tlen = 0 ;
	  simDisk[drv].tnum = 0 ;
	  free( simDisk[drv].info ) ;
	  free( simDisk[drv].buf  ) ;
	}
      }
    }
  }

  return(res) ;
}


int mount_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      mount_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


int umount_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      umount_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void mount_print_help()
{
  SYSERR(
	 "Mount disk device\n"
	 "Usage: mount [drv] [trk] [tnum]\n"
	 "  drv   : Drive number(0 to %d).\n"
	 "  trk   : Allocate track length.\n"
	 "  tnum  : Allocate track number.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     ディスクデバイスをマウントします。マウントしたディスクはディスクデバイス\n"
	 "    シミュレーションで利用可能となります。\n"
	 "     引き数を省略すると現在マウントされているディスクの一覧が表示されます。\n"
	 , SIM_DISK_DRIVE_NUM-1
	 ) ;
}


void umount_print_help()
{
  SYSERR(
	 "Unmount disk device\n"
	 "Usage: umount drv\n"
	 "  drv   : Drive number(0 to %d).\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     指定したドライブのディスクデバイスをアンマウントします。\n"
	 , SIM_DISK_DRIVE_NUM-1
	 ) ;
}
