/**********************************************************************
  Passポイントの設定
  通過点を指定し、通った場合はメッセージを出力する
  $Id: pp_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"

int  pp_parse_opt(struct GETOPT_ARG *args) ;
void pp_print_help() ;

int pp_cmd( struct GETOPT_ARG *args , int mode )
{
  int i,j ;
  int id ; 
  char *label ; 
  int ret ;
  
  ret=pp_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  if( XGETOPT_REFARGP(args,1)==NULL ){
    for( i=0 ; i<MAX_PROCESS_ID ; i++ ){
      if( proc_list.proc[i].pp_num>0 ){
	SYSMSG("--- Pass Point(s) for Process ID = %2d --\n",i) ;
	for( j=0 ; j<proc_list.proc[i].pp_num ; j++ ){
	  SYSMSG("No.%02d : 0x%08x %s\n",
		 proc_list.proc[i].pp[j].id,
		 proc_list.proc[i].pp[j].ad,
		 proc_list.proc[i].pp[j].label) ;
	}
      }
    }
    return(NORMAL_EXIT) ;
  }
  if( XGETOPT_REFARGP(args,2)==NULL ){
    id=proc_list.cur_id ;
  }else{
    if( XGETOPT_REFARGP(args,2)[0]=='+' ){
      id=proc_list.cur_id+1 ;
    }else if( XGETOPT_REFARGP(args,2)[0]=='-' ){
      id=proc_list.cur_id-1 ;
    }else if( XGETOPT_REFARGP(args,2)[0]=='=' ){
      id=proc_list.cur_id ;
    }else{
      id=exatoi(XGETOPT_REFARGP(args,2)) ;
    }
    if( id>=MAX_PROCESS_ID || id<0 ){
      SYSMSG("Invalid Process ID (Min=0,Max=%d specify=%d).\n",
	     (MAX_PROCESS_ID)-1,id) ;
      return(NORMAL_EXIT) ;
    }
  }
  if( XGETOPT_REFARGP(args,3)!=NULL ){
    label=malloc( strlen(XGETOPT_REFARGP(args,3))+1 ) ;
    strcpy(label,XGETOPT_REFARGP(args,3)) ;
  }else{
    label=malloc( 1 ) ;
    label[0]='\0' ;
  }
  
  if( proc_list.proc[id].pp_num==MAX_PASS_POINT_NUM ){
    SYSMSG("Pass Point Buffer is full.\n") ;
    return(NORMAL_EXIT) ;
  }
  
  proc_list.proc[id].pp[proc_list.proc[id].pp_num].ad = exatoi(XGETOPT_REFARGP(args,1)) ;
  proc_list.proc[id].pp[proc_list.proc[id].pp_num].id = proc_list.proc[id].pp_id++ ;
  proc_list.proc[id].pp[proc_list.proc[id].pp_num].label=label ;
  SYSMSG("Setting pp=0x%08x , BPID=%d , label=%s , PID=%d\n",
	 proc_list.proc[id].pp[proc_list.proc[id].pp_num].ad,
	 proc_list.proc[id].pp[proc_list.proc[id].pp_num].id,
	 proc_list.proc[id].pp[proc_list.proc[id].pp_num].label,
	 id) ;
  proc_list.proc[id].pp_num++ ;
  return(NORMAL_EXIT) ;

}


int pp_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      pp_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void pp_print_help()
{
  SYSERR(
	 "Set PassPoint\n"
	 "Usage: pp [Options] adrs [[pid|+|-|=] [label]]\n"
	 "  adrs : setting address.\n"
	 "  pid  : set pid.\n"
	 "   +   : set current pid+1.\n"
	 "   -   : set current pid-1.\n"
	 "   =   : set current pid.\n"
	 "  label: set pp name.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     通過を監視したいアドレスを設定します。設定できるアドレスは1プロセスID\n"
	 "    につき256個まで指定できます。\n"
	 "     指定されたアドレスを通過するとppmsgで指定した出力フォーマットで、通過し\n"
	 "    た旨を示すメッセージが表示されます。\n"
	 "\n"
	 "     パスアドレスの指定を省略した場合は、設定されている全パスアドレスの一覧\n"
	 "    を表示します。\n"
	 ) ;
}
