/**********************************************************************
  ppmsgコマンド PassPoint通過時の出力メッセージ形式を選択
  $Id: ppmsg_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"

int  ppmsg_parse_opt(struct GETOPT_ARG *args) ;
void ppmsg_print_help() ;

int ppmsg_cmd( struct GETOPT_ARG *args , int mode )
{
  static char *msg[]={"only Dump PC",
		      "Dump GPR and others",
		      "Dump all registers",
		      "Dump GPRn",
		      "Dump FPRn",
		      "Dump SPRn",
		      "Dump TR"} ;
  int ret ;
  
  ret=ppmsg_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  if( XGETOPT_REFARGP(args,1)==NULL ){
    SYSMSG("Current PassPoint message format is %s\n", msg[ppmsg_stat.msg_type] ) ;
    return(NORMAL_EXIT) ;
  }
  if(       strcmp(XGETOPT_REFARGP(args,1),"pc" )==0 ){
    ppmsg_stat.msg_type  =PP_DUMP_PC ;
    ppmsg_stat.print_type=PRINT_NON  ;
  }else if( strcmp(XGETOPT_REFARGP(args,1),"gpr")==0 ){
    if( XGETOPT_REFARGP(args,2)==NULL ){
      ppmsg_stat.msg_type  =PP_DUMP_GPR ;
      ppmsg_stat.print_type=(PRINT_CRS|PRINT_GPRS) ;
    }else{
      int regno ;
      regno=exatoi(XGETOPT_REFARGP(args,2)) ;
      if( regno<0 || regno>31 ){
	SYSMSG("Invalid GPR No. %d\n",regno ) ;
	return(NORMAL_EXIT) ;
      }
      ppmsg_stat.msg_type  =PP_DUMP_GPR ;
      ppmsg_stat.print_type=(PRINT_GPRn|regno) ;
    }
  }else if( strcmp(XGETOPT_REFARGP(args,1),"all")==0 ){
    ppmsg_stat.msg_type  =PP_DUMP_ALL ;
    ppmsg_stat.print_type=(PRINT_CRS|PRINT_GPRS|PRINT_FPRS) ;
  }else if( strcmp(XGETOPT_REFARGP(args,1),"tr")==0 ){
    ppmsg_stat.msg_type  =PP_DUMP_TR ;
    ppmsg_stat.print_type=(PRINT_CRS|PRINT_GPRS|PRINT_FPRS|PRINT_TRS) ;
  }else{
    SYSMSG("Invalid argument %s\n",XGETOPT_REFARGP(args,1) ) ;
  }
  
  return(NORMAL_EXIT) ;
}


int ppmsg_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      ppmsg_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void ppmsg_print_help()
{
  SYSERR(
	 "setting PassPoint message\n"
	 "Usage: ppmsg [Options] type\n"
	 " type\n"
	 "  pc    : print PC and Instruction.\n"
	 "  gpr   : print GPR and others.\n"
	 "  all   : print more infomation.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     ppコマンドで設定したパスアドレス通過時の出力フォーマットを指定します。\n"
	 "\n"
	 "      pc  : 通過した命令の逆アセンブルリストを表示します。\n"
	 "      gpr : 通過した命令の逆アセンブルリストと命令実行前のGPRおよびフラグ\n"
	 "            レジスタ類を表示します。\n"
	 "      all : 通過した命令の逆アセンブルリストと命令実行前のGPRより多くの\n"
	 "            レジスタの内容を表示します。\n"
	 "\n"
	 "    引き数を省略した場合は現在の出力フォーマットを示すメッセージを表示します。\n"
	 ) ;
}
