/**********************************************************************
  各種値の設定コマンド
  $Id: set_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_scemu.h"
#include "set_def.h"

int  set_mem( struct MACHINE_RESOURCE *mres, int, char *, int ) ;
int  set_parse_opt(struct GETOPT_ARG *args) ;
void set_print_help() ;

int set_cmd( struct GETOPT_ARG *args , int mode )
{
  struct	TARGET_TBL_STRUCT{
    char  *name ;
    int   flag ;
  } ;
  static struct TARGET_TBL_STRUCT trg_tbl[] = {
#include "set_tbl.h"
    { NULL			, 0 } /*** テーブル終端 ***/
  } ;
  
  int i ;
  int flag ;
  int id ;
  char *scemuname[]={ "newlibPowerX", "LinuxPPC" } ;
  int    ret ;
  
  ret=set_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  id=proc_list.cur_id ;
  
  if( XGETOPT_REFARGP(args,1)==NULL || XGETOPT_REFARGP(args,2)==NULL ){
    SYSMSG("HeapTop   = 0x%08x\n",proc_list.proc[id].heap_top) ;
    SYSMSG("HeapSize  = 0x%08x\n",proc_list.proc[id].heap_size) ;
    SYSMSG("StartPC   = 0x%08x\n",default_exec_start) ;
    SYSMSG("StackTop  = 0x%08x\n",proc_list.proc[id].mem_size[TEXT_SEG_TOP] - default_stack_top_offset) ;
    SYSMSG("SCemuMode = %s\n",    scemuname[sc_emu_mode]) ;
    return(NORMAL_EXIT) ;
  }
  
  for( i=0 ; ; i++ ){
    if( trg_tbl[i].name==NULL ){
      SYSMSG("Specified invalid argument(%s)\n",XGETOPT_REFARGP(args,1)) ;
      return(SPECIFY_ERROR) ;
    }
    if( strncmp(XGETOPT_REFARGP(args,1),"spr",3)==0 ){
      flag=SET_SPR0+ exatoi( &(XGETOPT_REFARGP(args,1))[3] ) ;
      break ;
    }else if( strcmp(XGETOPT_REFARGP(args,1),trg_tbl[i].name)==0 ){
      flag=trg_tbl[i].flag ;
      break ;
    }
  }
  
  if( flag==SET_PC ){                                    /** PCの設定 **/
    proc_list.proc[id].pc = exatoi(XGETOPT_REFARGP(args,2)) ;
    iexe_count  = 0 ;
    SYSMSG("pc=%x and Reset executed instruction count.\n",proc_list.proc[id].pc) ;
    return(NORMAL_EXIT) ;
  }
  if( flag==SET_XER ){                                   /** XERの設定 **/
    proc_list.proc[id].FLGR.xer = exatoi(XGETOPT_REFARGP(args,2)) ;
    SYSMSG("exr=%x\n",proc_list.proc[id].FLGR.xer) ;
    return(NORMAL_EXIT) ;
  }
  if( flag>=SET_R0 && flag<=SET_R31 ){                   /** GPRの設定 **/
    proc_list.proc[id].GPR[flag-SET_R0].rl=(unsigned int)exatoi(XGETOPT_REFARGP(args,2)) ;
    SYSMSG("r%d=%x\n",flag-SET_R0,proc_list.proc[id].GPR[flag-SET_R0].rl) ;
    return(NORMAL_EXIT) ;
  }
  if( flag>=SET_F0 && flag<=SET_F31 ){                   /** FPRの設定 **/
    proc_list.proc[id].FPR[flag-SET_F0].f=(unsigned int)exatoi(XGETOPT_REFARGP(args,2)) ;
    SYSMSG("f%d=%f\n",flag-SET_F0,proc_list.proc[id].FPR[flag-SET_F0].f) ;
    return(NORMAL_EXIT) ;
  }
  if( flag>=SET_SPR0 && flag<=SET_SPR1023 ){             /** SPRの設定 **/
    *proc_list.proc[id].SPR[flag-SET_SPR0]=(unsigned int)exatoi(XGETOPT_REFARGP(args,2)) ;
    SYSMSG("spr%d=0x%08x\n",flag-SET_SPR0,*proc_list.proc[id].SPR[flag-SET_SPR0]) ;
    return(NORMAL_EXIT) ;
  }
  if( flag==SET_MEM ){                                   /** メモリの設定 **/
    int	count ;
    int	st_adrs ;
    
    st_adrs=exatoi(XGETOPT_REFARGP(args,2)) ;
    if( lad(st_adrs)>=proc_list.proc[id].mem_size[seg(st_adrs)] ){
      SYSMSG("Start memory address is invalid.\n" ) ;
      return(SPECIFY_ERROR) ;
    }
    if( XGETOPT_REFARGP(args,3)==NULL ){
      SYSMSG("Data is not specified\n") ;
      return(SPECIFY_ERROR) ;
    }
    if( XGETOPT_REFARGP(args,4)==NULL ) count=1 ;
    else{
      count=exatoi(XGETOPT_REFARGP(args,4)) ;
      if( count<=0 ){
	SYSMSG("Invalid count is specified\n") ;
	return(SPECIFY_ERROR) ;
      }
    }
    set_mem( &proc_list.proc[id], st_adrs, XGETOPT_REFARGP(args,3), count ) ;
    return(NORMAL_EXIT) ;
  }
  if( flag==SET_HEAPTOP ){ /** HEAP先頭アドレスの設定 **/
    proc_list.proc[id].heap_top  = exatoi(XGETOPT_REFARGP(args,2)) ;
    proc_list.proc[id].heap_cur  = 0 ;
    SYSMSG("heap_top=0x%08x and size is reseted.\n",proc_list.proc[id].heap_top) ;
    return(NORMAL_EXIT) ;
  }
  if( flag==SET_HEAPTOP ){ /** HEAP size の設定 **/
    proc_list.proc[id].heap_size = default_heapsize ;
    SYSMSG("heap_size=0x%08x\n",proc_list.proc[id].heap_size) ;
    return(NORMAL_EXIT) ;
  }
  if( flag==SET_SPC ){ /** exec スタートPCの変更 **/
    default_exec_start=exatoi(XGETOPT_REFARGP(args,2)) ;
    SYSMSG("start_pc=0x%08x\n",default_exec_start) ;
    return(NORMAL_EXIT) ;
  }
  
  return(NORMAL_EXIT) ;
}


int set_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      set_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void set_print_help()
{
  SYSERR(
	 "set memory/register/etc value\n"
	 "Usage: set [Options] [type val [rep]]\n"
	 " type\n"
	 "  r0-31     : specify GPRx value\n"
	 "  f0-31     : specify FPRx value\n"
	 "  spr0-1023 : specify SPRx value\n"
	 "  pc        : specify PC  value\n"
	 "  xer       : specify XER  value\n"
	 "  mem       : specify memory value\n"
	 "  ht        : specify HeapTop address\n"
	 "  hs        : specify HeapTop size\n"
	 "  spc       : specify exec() start pc\n"
	 "  ex        : specify exception status\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     指定のレジスタ及びメモリアドレスに値をセットします。\n"
	 "    レジスタを指定した場合、指定した値がレジスタにセットされます。\n"
	 "    メモリを指定した場合、書き込み開始アドレスに1Byte単位で与えた値を書き込\n"
	 "    みます。繰り返し回数を指定できます。\n"
	 "     省略時はシミュレーション実行時設定の一覧を表示します。\n"
	 ) ;
}
