/**********************************************************************
  fork エミュレーション
  $Id: sim_fork.c 77 2011-11-25 15:16:46Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"
#include "op_scemu.h"

int	sim_fork( struct PROCESS_LIST *proc_list )
{
  int child_id ;
  int parent_id ; 
  struct MACHINE_RESOURCE *child_mres ;
  struct MACHINE_RESOURCE *parent_mres ;
  struct point_struct bp[MAX_BREAK_POINT_NUM] ; 
  int                 bp_id        ;
  int                 bp_num       ;
  struct point_struct pp[MAX_PASS_POINT_NUM] ;
  int                 pp_id        ;
  int                 pp_num       ;
  int i ;
  int ret ;
  
  parent_id = proc_list->cur_id ;
  proc_list->cur_id++ ;
  child_id  = proc_list->cur_id ;

  for( i=0 ; i<MAX_BREAK_POINT_NUM ; i++ ){
    bp[i]=proc_list->proc[child_id].bp[i] ;
  }
  for( i=0 ; i<MAX_PASS_POINT_NUM ; i++ ){
    pp[i]=proc_list->proc[child_id].pp[i] ;
  }
  bp_id =proc_list->proc[child_id].bp_id  ;
  bp_num=proc_list->proc[child_id].bp_num ;
  pp_id =proc_list->proc[child_id].pp_id  ;
  pp_num=proc_list->proc[child_id].pp_num ;
  
  proc_list->proc[child_id]=proc_list->proc[parent_id] ;
  ret=dup_mman( &proc_list->proc[parent_id], &proc_list->proc[child_id] ) ;
  if( ret!=NORMAL_EXIT ) return(MEM_ALLOC_ERROR) ;

  for( i=0 ; i<MAX_BREAK_POINT_NUM ; i++ ){
    proc_list->proc[child_id].bp[i]=bp[i] ;
  }
  for( i=0 ; i<MAX_PASS_POINT_NUM ; i++ ){
    proc_list->proc[child_id].pp[i]=pp[i] ;
  }
  proc_list->proc[child_id].bp_id =bp_id  ;
  proc_list->proc[child_id].bp_num=bp_num ;
  proc_list->proc[child_id].pp_id =pp_id  ;
  proc_list->proc[child_id].pp_num=pp_num ;

  child_mres =&proc_list->proc[child_id] ;
  parent_mres=&proc_list->proc[parent_id] ;
  
  child_mres->SPR[1] = &child_mres->FLGR.xer  ; // XER
  child_mres->SPR[8] = &child_mres->FLGR.lrl  ; // LR
  child_mres->SPR[9] = &child_mres->FLGR.ctrl ; // CTR

  for( i=0 ; i<MEM_SEGMENT_NUM ; i++ ){
    if( parent_mres->mem_size[i]!=0 &&
	parent_mres->mem_attr[i]!=MEM_ATTR_SHARED ){
      child_mres->mem[i]=NULL ;
      child_mres->mem_size[i]=0 ;
      ret=memal( child_mres, i, parent_mres->mem_size[i], parent_mres->mem_attr[i]) ;
      if( ret!=NORMAL_EXIT ){
	SYSMSG("sim_fork memory allocate error.\n") ;
	//need free all child mem
	return(MEM_ALLOC_ERROR) ;
      }
      memcpy( child_mres->mem[i], parent_mres->mem[i], parent_mres->mem_size[i] ) ;
    }
  }
  
  child_mres->id = child_id ;

  for( i=0 ; i<MAX_OPEN_FD ; i++ ){/** 全fildesをコピーする **/
    child_mres->openfd[i]=-1 ; /** 親プロセスと全く同じfildesを持っているのでinvalidateする **/
    if( parent_mres->openfd[i]!=0 ){
      set_phyfd2(child_mres,i,dup(parent_mres->openfd[i])) ;
    }
  }
  
  return(NORMAL_EXIT) ;
}

/**************************************************************************
 メモ(2002/05/15)

 bp/pp情報をプロセス構造体に入れてしまったため、forkでプロセスコピーした時
 にこれらもコピーされてしまう不具合が発覚した。bp/pp情報をプロセス情報の外
 に出すのが正解だが、そこら中 修正する必要があるため、プロセスコピーする
 前にbp/pp情報を保存しておいて、プロセスコピー後に元あったbp/pp情報に戻す
 方法にする。うーん、失敗でした.........
**************************************************************************/
