/**********************************************************************
  mmapシステムコールエミュレーション用メモリマネージャ
   for glibc based C library
  $Id: sim_mman.c 77 2011-11-25 15:16:46Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"
#include "op_scemu.h"

#define BLKSIZE 4096
#define UNUSE      0
#define INUSE      1

int init_mman( struct MACHINE_RESOURCE *mres )
{
#ifdef SUPPORT_SYSTEMCALL_EMULATION
  int size ;
  int i ;

  size=(mres->heap_size / BLKSIZE) + ((mres->heap_size%BLKSIZE)==0?0:1) ;
  mres->mman.map=malloc(size) ;
  if( mres->mman.map==NULL ){
    return(MEM_ALLOC_ERROR) ;
  }

  mres->mman.size=size ;
  mres->mman.htop=mres->heap_top ;
  for( i=0 ; i<mres->mman.size ; i++ ){
    mres->mman.map[i]=UNUSE ;
  }
#endif
  return( NORMAL_EXIT ) ;
}

int release_mman( struct MACHINE_RESOURCE *mres )
{
#ifdef SUPPORT_SYSTEMCALL_EMULATION
  free(mres->mman.map) ;
  mres->mman.size=0 ;
#endif
  return( NORMAL_EXIT ) ;
}

int dup_mman( struct MACHINE_RESOURCE *mres_s, struct MACHINE_RESOURCE *mres_d )
{
#ifdef SUPPORT_SYSTEMCALL_EMULATION
  int i ;
  int ret ;
  
  ret=init_mman( mres_d ) ;
  if( ret!=NORMAL_EXIT ) return(ret) ;
  
  for( i=0 ; i<mres_s->mman.size ; i++ ){
    mres_d->mman.map[i]=mres_s->mman.map[i] ;
  }
  
#endif
  return( NORMAL_EXIT ) ;
}

int sim_mmap(struct MACHINE_RESOURCE *mres,int _adr,int _len,int _prot,int _flags,int _fd,int _off)
{
  int size ;
  int count ;
  int found ;
  int top ;
  int i ;
  
  if( _adr!=0 ){
    return(0xffffffff) ;
  }

  size=(_len/BLKSIZE)+((_len%BLKSIZE)==0?0:1) ;

  count=0 ;
  found=0 ;
  top=0 ;
  for( i=0 ; i<mres->mman.size ; i++ ){
    if( mres->mman.map[i]==UNUSE ){
      if( count==0 ) top=i ;
      count++ ;
      if( count==size ){
	found=1 ;
	break ;
      }
    }else{
      count=0 ;
    }
  }
  if( found!=0 ){
    for( i=0 ; i<size ; i++ ){
      mres->mman.map[top+i]=INUSE ;
    }
    return( mres->mman.htop+(top*BLKSIZE) ) ;
  }else{
    return(0xffffffff) ;
  }

}
	  
int sim_munmap(struct MACHINE_RESOURCE *mres,int _adr,int _len)
{
  int size ;
  int top ;
  int i ;
    
  size=(_len/BLKSIZE)+((_len%BLKSIZE)==0?0:1) ;
  top =(_adr - mres->mman.htop)/BLKSIZE ;

  for( i=0 ; i<size ; i++ ){
    mres->mman.map[top+i]=UNUSE ;
  }
  return(0) ;
}

int sim_munmap_all(struct MACHINE_RESOURCE *mres)
{
  int i ;
    
  for( i=0 ; i<mres->mman.size ; i++ ){
    mres->mman.map[i]=UNUSE ;
  }
  return(0) ;
}


