/**********************************************************************
  実行命令サマリ表示
  $Id: sum_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"

int  sum_parse_opt(struct GETOPT_ARG *args) ;
void sum_print_help() ;

int sum_cmd( struct GETOPT_ARG *args , int mode )
{
  int i,j ;
  long long int sum ;
  char tmpbuf[32] ;
  int ret ;
  
  ret=sum_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  if( XGETOPT_REFARGP(args,1)!=NULL ){
    if( strcmp(XGETOPT_REFARGP(args,1),"reset")==0 ){
      reset_exec_op_count() ;
      return(NORMAL_EXIT) ;
    }else{
      SYSMSG("Unknown subcommand (%s).\n",XGETOPT_REFARGP(args,1)) ;
      return(NORMAL_EXIT) ;
    }
  }
    
  sum=0 ;
  for( j=0 ; j<=1 ; j++ ){
    for( i=0 ; i<TBL_SIZE_bit00to05 ; i++ ){
      if( op_func_tbl[i].count!=0 ){
	if( i==0x13 || i==0x1f || i==0x3b || i==0x3f ) continue ;
	if( j==1 ){
	  SYSMSG("%-12s = ",op_func_tbl[i].name) ;
	  SYSMSGLL(op_func_tbl[i].count,12) ;
	  sprintf(tmpbuf,"   %3.6f %%",(double)op_func_tbl[i].count/sum*100.0) ;
	  SYSMSG("%14s",tmpbuf) ;
	  SYSMSG("\n") ;
	}else{
	  sum+=op_func_tbl[i].count ;
	}
      }
    }
    for( i=0 ; i<TBL_SIZE_bit21to30 ; i++ ){
      if( op_13_subfunc_tbl[i].count!=0 ){
	if( j==1 ){
	  SYSMSG("%-12s = ",op_13_subfunc_tbl[i].name) ;
	  SYSMSGLL(op_13_subfunc_tbl[i].count,12) ;
	  sprintf(tmpbuf,"   %3.6f %%",(double)op_13_subfunc_tbl[i].count/sum*100.0) ;
	  SYSMSG("%14s",tmpbuf) ;
	  SYSMSG("\n") ;
	}else{
	  sum+=op_13_subfunc_tbl[i].count ;
	}
      }
    }
    for( i=0 ; i<TBL_SIZE_bit21to30 ; i++ ){
      if( op_1f_subfunc_tbl[i].count!=0 ){
	if( j==1 ){
	  SYSMSG("%-12s = ",op_1f_subfunc_tbl[i].name) ;
	  SYSMSGLL(op_1f_subfunc_tbl[i].count,12) ;
	  sprintf(tmpbuf,"   %3.6f %%",(double)op_1f_subfunc_tbl[i].count/sum*100.0) ;
	  SYSMSG("%14s",tmpbuf) ;
	  SYSMSG("\n") ;
	}else{
	  sum+=op_1f_subfunc_tbl[i].count ;
	}
      }
    }
    for( i=0 ; i<TBL_SIZE_bit27to30 ; i++ ){
      if( op_3b_subfunc_tbl[i].count!=0 ){
	if( j==1 ){
	  SYSMSG("%-12s = ",op_3b_subfunc_tbl[i].name) ;
	  SYSMSGLL(op_3b_subfunc_tbl[i].count,12) ;
	  sprintf(tmpbuf,"   %3.6f %%",(double)op_3b_subfunc_tbl[i].count/sum*100.0) ;
	  SYSMSG("%14s",tmpbuf) ;
	  SYSMSG("\n") ;
	}else{
	  sum+=op_3b_subfunc_tbl[i].count ;
	}
      }
    }
    for( i=0 ; i<TBL_SIZE_bit21to30 ; i++ ){
      if( op_3f_subfunc0_tbl[i].count!=0 ){
	if( j==1 ){
	  SYSMSG("%-12s = ",op_3f_subfunc0_tbl[i].name) ;
	  SYSMSGLL(op_3f_subfunc0_tbl[i].count,12) ;
	  sprintf(tmpbuf,"   %3.6f %%",(double)op_3f_subfunc0_tbl[i].count/sum*100.0) ;
	  SYSMSG("%14s",tmpbuf) ;
	  SYSMSG("\n") ;
	}else{
	  sum+=op_3f_subfunc0_tbl[i].count ;
	}
      }
    }
    for( i=0 ; i<TBL_SIZE_bit27to30 ; i++ ){
      if( op_3f_subfunc1_tbl[i].count!=0 ){
	if( j==1 ){
	  SYSMSG("%-12s = ",op_3f_subfunc1_tbl[i].name) ;
	  SYSMSGLL(op_3f_subfunc1_tbl[i].count,12) ;
	  sprintf(tmpbuf,"   %3.6f %%",(double)op_3f_subfunc1_tbl[i].count/sum*100.0) ;
	  SYSMSG("%14s",tmpbuf) ;
	  SYSMSG("\n") ;
	}else{
	  sum+=op_3f_subfunc1_tbl[i].count ;
	}
      }
    }
  }
  
  SYSMSG("Total_count  = ") ;
  SYSMSGLL(sum,12) ;
  SYSMSG("\n") ;
  
  return(NORMAL_EXIT) ;
}


int sum_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      sum_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void sum_print_help()
{
  SYSERR(
	 "Print executed instruction summary\n"
	 "Usage: sum\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     execもしくはgoコマンドで実行された命令列の実行された回数を命令毎に\n"
	 "    サマリ表示します。\n"
	 ) ;
}
