/**********************************************************************
  traceコマンド
  $Id: trace_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"

int  trace_parse_opt(struct GETOPT_ARG *args) ;
void trace_print_help() ;

int trace_cmd( struct GETOPT_ARG *args , int mode )
{
  static char *msg[]={"Trace OFF","Trace PC","Trace GPR and others",
		      "Trace all Registers"} ;
  int ret ;
  
  ret=trace_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;

  if( XGETOPT_REFARGP(args,1)==NULL ){
    SYSMSG("Current tracing state=%s\n",msg[trace_stat.msg_type] ) ;
    return(NORMAL_EXIT) ;
  }
  if(       strcmp(XGETOPT_REFARGP(args,1),"pc" )==0 ){
    trace_stat.msg_type  =TRACE_PC  ;
    trace_stat.print_type=PRINT_NON ;
  }else if( strcmp(XGETOPT_REFARGP(args,1),"gpr")==0 ){
    trace_stat.msg_type  =TRACE_GRP ;
    trace_stat.print_type=(PRINT_CRS|PRINT_GPRS) ;
  }else if( strcmp(XGETOPT_REFARGP(args,1),"all")==0 ){
    trace_stat.msg_type  =TRACE_ALL ;
    trace_stat.print_type=(PRINT_CRS|PRINT_GPRS|PRINT_FPRS) ;
  }else if( strcmp(XGETOPT_REFARGP(args,1),"off")==0 ){
    trace_stat.msg_type  =TRACE_OFF ;
    trace_stat.print_type=PRINT_NON ;
  }else{
    SYSMSG("Invalid argument %s\n",XGETOPT_REFARGP(args,1) ) ;
  }
  
  return(NORMAL_EXIT) ;
}


int trace_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      trace_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void trace_print_help()
{
  SYSERR(
	 "trace execution simulate\n"
	 "Usage: trace [Options] type\n"
	 " type\n"
	 "  pc    : print PC and Instruction.\n"
	 "  gpr   : print GPR and others.\n"
	 "  all   : print more infomation.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     go,exec,step,c コマンドの実行時に命令実行前のレジスタ状態や実行命令の\n"
	 "    逆アセンブルリストを表示します。\n"
	 "\n"
	 "      pc  : 通過した命令の逆アセンブルリストを表示します。\n"
	 "      gpr : 通過した命令の逆アセンブルリストと命令実行前のGPRおよびフラグ\n"
	 "            レジスタ類を表示します。\n"
	 "      all : 通過した命令の逆アセンブルリストと命令実行前のgprより多くの\n"
	 "            レジスタの内容を表示します。\n"
	 "      off : 実行時表示をoffにします。\n"
	 "\n"
	 "    引き数を省略した場合は現在の出力フォーマットを示すメッセージを表示します。\n"
	 ) ;
}
