/**********************************************************************
  trussライクコマンド
  sc命令実行時の情報を表示のフラグを制御する
  $Id: truss_cmd.c 78 2011-11-25 15:46:20Z tane $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"

int  truss_parse_opt(struct GETOPT_ARG *args) ;
void truss_print_help() ;

int truss_cmd( struct GETOPT_ARG *args, int mode )
{
  int ret ;

  ret=truss_parse_opt(args) ;
  if( ret<0 ) return(NORMAL_EXIT) ;
  
  if( XGETOPT_REFARGP(args,1)==NULL ){
    SYSMSG("Current truss state=%s\n",truss_stat==ON ? "ON" : "OFF" ) ;
    return(NORMAL_EXIT) ;
  }
  if( strcmp(XGETOPT_REFARGP(args,1),"on")==0 ){
    truss_stat=ON ;
  }else if( strcmp(XGETOPT_REFARGP(args,1),"off")==0 ){
    truss_stat=OFF ;
  }else{
    SYSMSG("Invalid argument %s\n",XGETOPT_REFARGP(args,1) ) ;
  }
  
  return(NORMAL_EXIT) ;
}


int truss_parse_opt(struct GETOPT_ARG *args)
{
  static struct GETOPT_LIST getopt_list[]={
    {"help",               'h',  NONARG, 'h'},
    { NULL,               '\0',       0,'\0'} /*** terminator ***/
  } ;
  int ret ;
  
  while(-1){
    ret=xgetopt(args, getopt_list) ;
    if( ret==XGETOPT_PARSE_COMPLETE ) break ;
    switch(ret){
    case 'h' :
      truss_print_help() ;
      return(-1) ;
      break ;
    case XGETOPT_INVALID_OPTION :
      SYSERR("Invalid option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_ARG_NOT_FOUND :
      SYSERR("Illegal arg for option %s\n",xgetopt_err) ;
      return(-1) ;
      break ;
    case XGETOPT_NOT_OPTION :
      break ;
    default :
      SYSERR("xgetopt internal error\n") ;
      return(-1) ;
    }
  }
  return(0) ;
}


void truss_print_help()
{
  SYSERR(
	 "Trace system calls\n"
	 "Usage: truss [Options] [on|off]\n"
	 "  on   : Trace ON.\n"
	 "  off  : Trace OFF.\n"
	 " Options\n"
	 "  -h or --help : Print this help.\n"
	 "\n"
	 "     sc命令が実行され、システムコールがエミュレートされた時、システムコール\n"
	 "    に渡された引き数などを表示します。\n"
	 "    ppcsimの初期起動時には truss off の状態であり、sc命令が実行されてもメッ\n"
	 "    セージの表示は行われません。表示を行いたい場合は、goコマンドもしくは\n"
	 "    execコマンドの実行前に truss on を実行して下さい。\n"
	 "\n"
	 "    引き数を省略した場合は現在の出力ON/OFFのいずれの状態かを表示します。\n"
	 ) ;
}
  
